%% ============================= Hlavicka =================================
clc;
clear all;
close all;

%% ========================== Vstupni promenne ============================
% XDF soubor nahrany LSL - LabRecorder
soubor_dat = load_xdf('Proband_1.xdf');

% E&K algoritmus - parametr LAMBDA - stanoveni prahu pro rychlosti X/Y
lambda = 6;

% Stanoveni min. delky segmentu s mikrosakadou detekovanym E&K algoritmem
% pro dalsi analyzu
MIN_mikrosakada = 7;

%% ===================== Rozdeleni dat XDF souboru ========================
% EyeTracker
EyeTracker_zaznam = soubor_dat{1,1};
EyeTracker_data = soubor_dat{1,1}.time_series;

% rozdeleni 2_EEG
EEG_zaznam = soubor_dat{1,2};

%% ======================= Predzpracovani dat EEG =========================
% vytvoreni hlavicky EEG souboru pro FieldTrip
data = Uprava_EEG(soubor_dat,2);

% FieldTrip - cfg
cfg = [];
cfg.demean = 'yes';
cfg.detrend = 'yes';
cfg.lpfilttype = 'firws';
cfg.lpfilter = 'yes';
cfg.lpfreq = 100;
cfg.hpfilttype = 'firws';
cfg.hpfilter = 'yes';
cfg.hpfreq = 30;
cfg.dftfilter = 'yes';

% Predzpracovani EEG
EEG_data_preprocessing = ft_preprocessing(cfg,data);

disp('Predzpracovani EEG dat HOTOVO...')

%% ====================== Zpracovani dat EyeTracker =======================
% Urceni nahravaneho OKA
if (sum(EyeTracker_data(3,1:20))==0 || sum(EyeTracker_data(4,1:20))==0)
    EyeTracker_X = EyeTracker_data(1,:);
    EyeTracker_Y = EyeTracker_data(2,:);
    disp('Vybrano LEVE oko...')
else
    EyeTracker_X = EyeTracker_data(3,:);
    EyeTracker_Y = EyeTracker_data(4,:);
    disp('Vybrano PRAVE oko...')
end

% Nastaveni prvni hodnoty na 0
EyeTracker_X(1,1) = 0;
EyeTracker_Y(1,1) = 0;

% Nalezeni slepych hodnot v EyeTracker zaznamu
EyeTracker_hodnota_mimo = find(EyeTracker_X <= -32000);

EyeTracker_mimo_uprava = zeros(1,length(EyeTracker_hodnota_mimo)+1);
EyeTracker_mimo_uprava(1,1) = EyeTracker_hodnota_mimo(1,1);
EyeTracker_mimo_uprava(1,2:end) = EyeTracker_hodnota_mimo;

EyeTracker_mimo_zkracene = [];

j=1;
for i=1:(length(EyeTracker_mimo_uprava)-1)
    if (EyeTracker_mimo_uprava(1,i+1)-EyeTracker_mimo_uprava(1,i))~=1
        EyeTracker_mimo_zkracene(1,j) = EyeTracker_mimo_uprava(1,i);
        j = j + 1;
    end
end

EyeLink_X_interpol = EyeTracker_X;
EyeLink_Y_interpol = EyeTracker_Y;

% Nahrazeni slepych dat posledni znamou hodnotou
for i=1:length(EyeTracker_hodnota_mimo)
	EyeLink_X_interpol(1,EyeTracker_hodnota_mimo(1,i)) = EyeLink_X_interpol(1,EyeTracker_hodnota_mimo(1,i)-1);
    EyeLink_Y_interpol(1,EyeTracker_hodnota_mimo(1,i)) = EyeLink_Y_interpol(1,EyeTracker_hodnota_mimo(1,i)-1);
end

% Overeni odstraneni slepych hodnot ze zaznamu
Eye_hodnota_mimo_interpol = find(EyeLink_X_interpol <= -32000);

if isempty(Eye_hodnota_mimo_interpol)==1
    disp('ZADNY slepy vzorek v zaznamu...')
else 
    disp('POZOR! CHYBA! Slepy vzorek v zaznamu!!!')
end

disp('Predzpracovani Eye-Tracker dat HOTOVO...')

%% =========== Engbert - vlastni algoritmus na vypocet ====================
% Vektor rychlosti X/Y
vektor_rychlosti_X = [];
vektor_rychlosti_Y = [];

casovy_krok_Eye = 1/EyeTracker_zaznam.info.effective_srate;

% Vytvoreni vektoru rychlosti ze souradnic X/Y
for i=3:length(EyeLink_X_interpol)-2
    % vektory rychlosti pro interpolaci
    vektor_rychlosti_X(i) = (EyeLink_X_interpol(i+2) + EyeLink_X_interpol(i+1) - EyeLink_X_interpol(i-1)- EyeLink_X_interpol(i-2))/(6*casovy_krok_Eye);
    vektor_rychlosti_Y(i) = (EyeLink_Y_interpol(i+2) + EyeLink_Y_interpol(i+1) - EyeLink_Y_interpol(i-1)- EyeLink_Y_interpol(i-2))/(6*casovy_krok_Eye);
end

% Median rychlosti X/Y
Median_X = median(vektor_rychlosti_X);
Median_Y = median(vektor_rychlosti_Y);

% Vypocet sigmy pro rychlosti X/Y
sigma_X_2 = 0;
sigma_Y_2 = 0;
sigma_X = 0;
sigma_Y = 0;

for i=1:length(vektor_rychlosti_X)
    sigma_X_2 = (vektor_rychlosti_X(i)-Median_X)^2 + sigma_X_2;
    sigma_Y_2 = (vektor_rychlosti_Y(i)-Median_Y)^2 + sigma_Y_2;    
end

sigma_X = sqrt(sigma_X_2/(length(vektor_rychlosti_X)-1));
sigma_Y = sqrt(sigma_Y_2/(length(vektor_rychlosti_Y)-1));

% vypocet tresholdu
prah_X = lambda * sigma_X;
prah_Y = lambda * sigma_Y;

matice_prahy = zeros(2,length(vektor_rychlosti_X));

for i=1:length(vektor_rychlosti_X)
    if vektor_rychlosti_X(1,i) >= prah_X
        matice_prahy(1,i)=1;
    end
end

for i=1:length(vektor_rychlosti_Y)
    if vektor_rychlosti_Y(1,i) >= prah_Y 
        matice_prahy(2,i)=1;
    else
        matice_prahy(2,i)=0;
    end
end

Kontrola_nalez = sum(sum(matice_prahy));

if Kontrola_nalez == 0
    disp('Nenalezeny ZADNE mikrosakadicke artefakty...')
else
    disp('Nalezeny mikrosakadicke artefakty...')
end

% Funkce ELIPSA pro vykresleni oblasti prahu pro rychlosti X/Y
figure(1)
plot(vektor_rychlosti_X,vektor_rychlosti_Y,'y')
hold on
Elipsa = ellipse(Median_X,Median_Y,prah_X,prah_Y);
hold off
title('Graf vektor rychlost X/Y a hranice prahu E&K algoritmu')
ylabel('Rychlosti v ose Y (/ms)')
xlabel('Rychlosti v ose X (/ms)')

%% ================== Hledani nalezenych artefaktu ========================
% Vybrane hodnoty presahuji prah E&K algoritmu
rychlosti_mikrosakady_X = matice_prahy(1,:).*vektor_rychlosti_X;
rychlosti_mikrosakady_Y = matice_prahy(2,:).*vektor_rychlosti_Y;

% Zpetna rekonstrukce souradnic X/Y
souradnice_mikrosakady_X = zeros(1,length(rychlosti_mikrosakady_X));
souradnice_mikrosakady_Y = zeros(1,length(rychlosti_mikrosakady_Y));

for i=3:length(rychlosti_mikrosakady_X)-2
    if rychlosti_mikrosakady_X(i-2) ~= 0
        souradnice_mikrosakady_X(1,i-2:i+2) = EyeLink_X_interpol(1,i-2:i+2);
        souradnice_mikrosakady_Y(1,i-2:i+2) = EyeLink_Y_interpol(1,i-2:i+2);
    end
end

mikrosakady_X = find(souradnice_mikrosakady_X~=0);
mikrosakady_Y = find(souradnice_mikrosakady_Y~=0);

% Rozdeleni do segmentu obsahujici mikrosakadicky artefakt
matice_segmenty_X = [];
matice_casove_znacky = [];  
j = 1;
l = 1;

% Souradnice X
for i=1:length(mikrosakady_X)-1
    if (mikrosakady_X(i+1)-mikrosakady_X(i))==1
        matice_segmenty_X (j,l) = souradnice_mikrosakady_X(1,mikrosakady_X(i));
        matice_casove_znacky (j,l) = mikrosakady_X(i);
        l = l + 1;
    else
        matice_segmenty_X (j,l) = souradnice_mikrosakady_X(1,mikrosakady_X(i));
        matice_casove_znacky (j,l) = mikrosakady_X(i);
        j = j+1;
        l = 1;
    end
end

% Restartovani promenych
matice_segmenty_Y = [];
j = 1;
l = 1;

% Souradnice Y
for i=1:length(mikrosakady_Y)-1
    if (mikrosakady_Y(i+1)-mikrosakady_Y(i))==1
        matice_segmenty_Y (j,l) = souradnice_mikrosakady_Y(1,mikrosakady_Y(i));
        l = l + 1;
    else
        matice_segmenty_Y (j,l) = souradnice_mikrosakady_Y(1,mikrosakady_Y(i));
        j = j+1;
        l = 1;
    end
end

disp('Nalezeni mikrosakadickeho artefaktu v EEG zaznamu')

%% ================= Odstraneni kratkych segmentu =========================
% Prevedeni jednotlivych pozic samplu na cas pomoci efektivni vzorkovaci
% frekvence
casove_znacky_radek = length(matice_casove_znacky(:,1));
casove_znacky_sloupec = length(matice_casove_znacky(1,:));

rada_casovych_znacek = zeros(casove_znacky_radek,casove_znacky_sloupec);
rada_casovych_znacek = matice_casove_znacky/soubor_dat{1,1}.info.effective_srate;
rada_casovych_znacek = round(rada_casovych_znacek*1000);
rada_casovych_znacek(:,end+1:end+5) = 0;

% Stanoveni min delky segmentu - zbytek odstranen
vektor_delky_trialu = [];
rada_casovych_znacek_2 = [];

for i=1:length(rada_casovych_znacek(:,1))
    for j=1:length(rada_casovych_znacek(1,:))
        if rada_casovych_znacek(i,j) == 0
        rada_casovych_znacek(i,j+1:end) = 1;
        end
    end
end 

j = 1;
k = 1;

for i=1:length(rada_casovych_znacek(:,1))
    for j=1:length(rada_casovych_znacek(1,:))
        if rada_casovych_znacek(i,j) == 0
        vektor_delky_trialu(i,1) = abs(rada_casovych_znacek(i,1)-rada_casovych_znacek(i,j-1));
        end
    end
end 

j = 1;
for i=1:length(rada_casovych_znacek)
        if vektor_delky_trialu(i,1) >= MIN_mikrosakada
            rada_casovych_znacek_2(j,:) = rada_casovych_znacek(i,:);
            j = j+1;
        end
end

% Trialy s EEG segmenty - zacatek trialu
pocet_trialu_zacatek = length(rada_casovych_znacek_2(:,1));
zacatek_trialu = zeros(pocet_trialu_zacatek,1);
zacatek_trialu(:,1) = rada_casovych_znacek_2(:,1);

%% ================ Vytvoreni noveho souboru dat ze segmentu ==============
% Pocet trialu, ktere budou analyzovany dale
Puvodni_pocet_trialu = length(zacatek_trialu);

% Urceni sirky segmentu (ms)
pred_segmentem = 200;
po_segmentu = 199;
base_line = 400;
delka_segmentu = pred_segmentem + 1 + po_segmentu + base_line;

mikrosakady_EEG.trialy = [];

for i=0:Puvodni_pocet_trialu-1
    if (zacatek_trialu(i+1,1)-pred_segmentem) >=0 && (zacatek_trialu(i+1,1)+po_segmentu + base_line) <= length(data.trial{1,1})
        mikrosakady_EEG.trialy(:,i*delka_segmentu+1:i*delka_segmentu+delka_segmentu) = data.trial{1, 1}(:,(zacatek_trialu(i+1,1)-pred_segmentem):(zacatek_trialu(i+1,1) + po_segmentu + base_line));       
    end
end

% Regulace poctu trialu pri pripadnem odstraneni behem rekonstrukce
% segmentu v EEG zaznamu
Pocet_trialu = length(mikrosakady_EEG.trialy)/delka_segmentu;

%% ==================== Uprava vstupnich dat do ICA =======================
% Vytvoreni doplnujicich udaju pro vstup do funkce FieldTripu
mikrosakady_EEG.label = {'E3';'E4';'E5';'E6';'E7';'E8';'E9';'E10';'E11';'E12';'E13';'E14';'E15';'E16';'E17';'E18';'E19';'E20';'E21';'E22';'E23';'E24';'E25';'E26';'E27';'E28';'E29';'E30';'E31';'E32';'E33';'E34';'E35';'E36';'E37';'E38';'E39';'E40';'E41';'E42';'E43';'E44';'E45';'E46';'E47';'E48';'E49';'E50';'E51';'E52';'E53';'E54';'E55';'E56';'E57';'E58';'E59';'E60';'E61';'E62';'E63';'E64';'E65';'E66';'E67';'E68';'E69';'E70';'E71';'E72';'E73';'E74';'E75';'E76';'E77';'E78';'E79';'E80';'E81';'E82';'E83';'E84';'E85';'E86';'E87';'E88';'E89';'E90';'E91';'E92';'E93';'E94';'E95';'E96';'E97';'E98';'E99';'E100';'E101';'E102';'E103';'E104';'E105';'E106';'E107';'E108';'E109';'E110';'E111';'E112';'E113';'E114';'E115';'E116';'E117';'E118';'E119';'E120';'E121';'E122';'E123';'E124';'E125';'E126';'E127';'E128';'E129';'E130';'E131';'E132';'E133';'E134';'E135';'E136';'E137';'E138';'E139';'E140';'E141';'E142';'E143';'E144';'E145';'E146';'E147';'E148';'E149';'E150';'E151';'E152';'E153';'E154';'E155';'E156';'E157';'E158';'E159';'E160';'E161';'E162';'E163';'E164';'E165';'E166';'E167';'E168';'E169';'E170';'E171';'E172';'E173';'E174';'E175';'E176';'E177';'E178';'E179';'E180';'E181';'E182';'E183';'E184';'E185';'E186';'E187';'E188';'E189';'E190';'E191';'E192';'E193';'E194';'E195';'E196';'E197';'E198';'E199';'E200';'E201';'E202';'E203';'E204';'E205';'E206';'E207';'E208';'E209';'E210';'E211';'E212';'E213';'E214';'E215';'E216';'E217';'E218';'E219';'E220';'E221';'E222';'E223';'E224';'E225';'E226';'E227';'E228';'E229';'E230';'E231';'E232';'E233';'E234';'E235';'E236';'E237';'E238';'E239';'E240';'E241';'E242';'E243';'E244';'E245';'E246';'E247';'E248';'E249';'E250';'E251';'E252';'E253';'E254';'E255';'E256';'E257';'ECG'};
mikrosakady_EEG.fsample = 1000;
mikrosakady_EEG.time = 1:1:length(mikrosakady_EEG.trialy);                 
mikrosakady_EEG.fsampleinfo = [1,length(mikrosakady_EEG.trialy)];

mikrosakady_EEG.hdr.Fs = 1000;
mikrosakady_EEG.hdr.nChans = 256;
mikrosakady_EEG.hdr.nSamples = length(mikrosakady_EEG.trialy);
mikrosakady_EEG.hdr.nSamplesPre = 0;
mikrosakady_EEG.hdr.nTrials = 1;
mikrosakady_EEG.hdr.label = {'E3';'E4';'E5';'E6';'E7';'E8';'E9';'E10';'E11';'E12';'E13';'E14';'E15';'E16';'E17';'E18';'E19';'E20';'E21';'E22';'E23';'E24';'E25';'E26';'E27';'E28';'E29';'E30';'E31';'E32';'E33';'E34';'E35';'E36';'E37';'E38';'E39';'E40';'E41';'E42';'E43';'E44';'E45';'E46';'E47';'E48';'E49';'E50';'E51';'E52';'E53';'E54';'E55';'E56';'E57';'E58';'E59';'E60';'E61';'E62';'E63';'E64';'E65';'E66';'E67';'E68';'E69';'E70';'E71';'E72';'E73';'E74';'E75';'E76';'E77';'E78';'E79';'E80';'E81';'E82';'E83';'E84';'E85';'E86';'E87';'E88';'E89';'E90';'E91';'E92';'E93';'E94';'E95';'E96';'E97';'E98';'E99';'E100';'E101';'E102';'E103';'E104';'E105';'E106';'E107';'E108';'E109';'E110';'E111';'E112';'E113';'E114';'E115';'E116';'E117';'E118';'E119';'E120';'E121';'E122';'E123';'E124';'E125';'E126';'E127';'E128';'E129';'E130';'E131';'E132';'E133';'E134';'E135';'E136';'E137';'E138';'E139';'E140';'E141';'E142';'E143';'E144';'E145';'E146';'E147';'E148';'E149';'E150';'E151';'E152';'E153';'E154';'E155';'E156';'E157';'E158';'E159';'E160';'E161';'E162';'E163';'E164';'E165';'E166';'E167';'E168';'E169';'E170';'E171';'E172';'E173';'E174';'E175';'E176';'E177';'E178';'E179';'E180';'E181';'E182';'E183';'E184';'E185';'E186';'E187';'E188';'E189';'E190';'E191';'E192';'E193';'E194';'E195';'E196';'E197';'E198';'E199';'E200';'E201';'E202';'E203';'E204';'E205';'E206';'E207';'E208';'E209';'E210';'E211';'E212';'E213';'E214';'E215';'E216';'E217';'E218';'E219';'E220';'E221';'E222';'E223';'E224';'E225';'E226';'E227';'E228';'E229';'E230';'E231';'E232';'E233';'E234';'E235';'E236';'E237';'E238';'E239';'E240';'E241';'E242';'E243';'E244';'E245';'E246';'E247';'E248';'E249';'E250';'E251';'E252';'E253';'E254';'E255';'E256';'E257';'ECG'};
mikrosakady_EEG.hdr.chantype = {'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'eeg';'ecg'};
mikrosakady_EEG.hdr.chanunit = {'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'uV';'unknown'};

%% ========================== ICA -> runICA ===============================
% Komponenty nastaveny na 35 - vyuzivano pri analyze signalu v praxi
cfg.numcomponent = 35;
cfg.method = 'runica';
cfg.demean = 'yes';

comp = ft_componentanalysis(cfg,mikrosakady_EEG);

%% ============= Identifikace komponent obsahujici artefakt ===============
rozlozeni_elektrod = load('GSN-HydroCel-256.mat');
cfg = [];
cfg.layout = rozlozeni_elektrod.lay;
cfg.viewmode = 'component';
cfg.blocksize = length(comp.time)/Puvodni_pocet_trialu*3;

ft_databrowser(cfg, comp);

%% ================= Odstraneni vybranych komponent =======================
% Vybrane komponenty oznacene jako mikrosakadicky artefakt
cfg.component = [15];
Mikrosakady_trialy_ICA = ft_rejectcomponent(cfg, comp, mikrosakady_EEG);

disp('Odstraneny vybrane komponenty obsahujici mikrosakadicky artefakt...')

%% ========== Rozdeleni na segmenty s artefaktem a bez a baseline =========
% Segmenty neobsahujici mikrosakadicky artefakt pred ICA
baseline.trial = [];
for i=0:Pocet_trialu-1
    baseline.trial{1,i+1}(1:256,1:delka_segmentu/2) = mikrosakady_EEG.trialy(:,i*delka_segmentu+delka_segmentu/2+1:i*delka_segmentu+delka_segmentu);
end

% Segmenty neobsahujici mikrosakadicky artefakt po ICA
baseline_po_ICA.trial = [];
for i=0:Pocet_trialu-1
    baseline_po_ICA.trial{1,i+1}(1:256,1:delka_segmentu/2) = Mikrosakady_trialy_ICA.avg(:,i*delka_segmentu+delka_segmentu/2+1:i*delka_segmentu+delka_segmentu);
end

% Segmenty obsahujici mikrosakadicky artefakt PO odstraneni komponent
% oznacenych jako mikrosakadicky artefakt
Mikrosakada_do_POW_po_ICA.trial = [];
for i=0:Pocet_trialu-1
    Mikrosakada_po_ICA.trial{1,i+1}(1:256,1:delka_segmentu/2) = Mikrosakady_trialy_ICA.avg(:,i*delka_segmentu+1:i*delka_segmentu+delka_segmentu/2);
end

% Segmenty obsahujici mikrosakadicky artefakt PRED odstraneni komponent
% oznacenych jako mikrosakadicky artefakt
Mikrosakada_do_POW_pred_ICA.trial = [];
for i=0:Pocet_trialu-1
    Mikrosakada_pred_ICA.trial{1,i+1}(1:256,1:delka_segmentu/2) = mikrosakady_EEG.trialy(:,i*delka_segmentu+1:i*delka_segmentu+delka_segmentu/2);
end

%% ==================== Uprava vstupnich dat do POW =======================
% Segmenty neobsahujici mikrosakadicky artefakt pred ICA
Baseline_uprava_dat_pred_ICA = Uprava_dat_POW(baseline);
Baseline_do_POW_pred_ICA = Prumerovani_trialu(Baseline_uprava_dat_pred_ICA);

% Segmenty neobsahujici mikrosakadicky artefakt po ICA
Baseline_uprava_dat_po_ICA = Uprava_dat_POW(baseline_po_ICA);
Baseline_do_POW_po_ICA = Prumerovani_trialu(Baseline_uprava_dat_po_ICA);

% Segmenty obsahujici mikrosakadicky artefakt PO odstraneni komponent
% oznacenych jako mikrosakadicky artefakt
Mikrosakady_po_ICA_uprava_dat = Uprava_dat_POW(Mikrosakada_po_ICA);
Mikrosakady_po_ICA_do_POW = Prumerovani_trialu(Mikrosakady_po_ICA_uprava_dat);

% Segmenty obsahujici mikrosakadicky artefakt PRED odstraneni komponent
% oznacenych jako mikrosakadicky artefakt
Mikrosakady_pred_ICA_uprava_dat = Uprava_dat_POW(Mikrosakada_pred_ICA);
Mikrosakady_pred_ICA_do_POW = Prumerovani_trialu(Mikrosakady_pred_ICA_uprava_dat);


%% ======================= POW - baseline pred ICA ========================
% Vytvoreni vykonoveho spektra
Baseline_do_POW_pred_ICA.time = Baseline_do_POW_pred_ICA.time/1000;
Baseline_do_POW_pred_ICA.fsample = 1000;

cfg = [];
cfg.output       = 'pow'; %pow = powerspectrum
cfg.channel      = 'all';
cfg.method       = 'mtmfft';
cfg.taper        = 'hanning';
cfg.foi          = 0:500;                                         
POW_spektrum_pred_ICA = ft_freqanalysis(cfg, Baseline_do_POW_pred_ICA);


%% ======================= POW - baseline po ICA ==========================
% Vytvoreni vykonoveho spektra
Baseline_do_POW_po_ICA.time = Baseline_do_POW_po_ICA.time/1000;
Baseline_do_POW_po_ICA.fsample = 1000;

cfg = [];
cfg.output       = 'pow'; %pow = powerspectrum
cfg.channel      = 'all';
cfg.method       = 'mtmfft';
cfg.taper        = 'hanning';
cfg.foi          = 0:500;                                         
POW_spektrum_po_ICA = ft_freqanalysis(cfg, Baseline_do_POW_po_ICA);

%% ================ POW - segment s mikrosakadou po ICA ===================
% Vytvoreni vykonoveho spektra
Mikrosakady_po_ICA_do_POW.time = Mikrosakady_po_ICA_do_POW.time/1000;
Mikrosakady_po_ICA_do_POW.fsample = 1000;

cfg = [];
cfg.output       = 'pow';
cfg.channel      = 'all';
cfg.method       = 'mtmfft';
cfg.taper        = 'hanning';
cfg.foi          = 0:1:500;   
POW_spektrum_mikrosakada_po_ICA = ft_freqanalysis(cfg, Mikrosakady_po_ICA_do_POW);

%% =============== POW - segment s mikrosakadou pred ICA ==================
% Vytvoreni vykonoveho spektra
Mikrosakady_pred_ICA_do_POW.time = Mikrosakady_pred_ICA_do_POW.time/1000;
Mikrosakady_pred_ICA_do_POW.fsample = 1000;

cfg = [];
cfg.output       = 'pow';
cfg.channel      = 'all';
cfg.method       = 'mtmfft';
cfg.taper        = 'hanning';
cfg.foi          = 0:1:500;                                                                
POW_spektrum_mikrosakada_pred_ICA = ft_freqanalysis(cfg, Mikrosakady_pred_ICA_do_POW);

%% ========================== Frontalni oblast ============================
% Z EEG dat je vybrana pouze frontalni oblast - mikrosakadicky artefakt se
% nejvice projevu prave ve frontalni oblasti
kanaly_frontal = [1:8 10:43 46:51 54:58 61:64 68:69 194:198 202:207 210:215 220:224];

POW_spektrum_pred_ICA_frontal = mean(POW_spektrum_pred_ICA.powspctrm(kanaly_frontal,:));
POW_spektrum_po_ICA_frontal = mean(POW_spektrum_po_ICA.powspctrm(kanaly_frontal,:));
POW_spektrum_mikrosakada_po_ICA_frontal = mean(POW_spektrum_mikrosakada_po_ICA.powspctrm(kanaly_frontal,:));
POW_spektrum_mikrosakada_pred_ICA_frontal = mean(POW_spektrum_mikrosakada_pred_ICA.powspctrm(kanaly_frontal,:));

%% ========================= Grafy POW spektra ============================
% Vykresleni relativnich vykonovych spekter ve frontalni oblasti
spodni_frekvence = 28;
horni_frekvence = 62;
zvolena_frekvencni_oblast = find(POW_spektrum_pred_ICA.freq>spodni_frekvence & POW_spektrum_pred_ICA.freq<horni_frekvence);

Rel_POW_spektrum_baseline_pred_ICA_frontal = POW_spektrum_pred_ICA_frontal(1,zvolena_frekvencni_oblast);%./sum(sum(POW_spektrum_pred_ICA.powspctrm));
Rel_POW_spektrum_baseline_po_ICA_frontal = POW_spektrum_po_ICA_frontal(1,zvolena_frekvencni_oblast);%./sum(sum(POW_spektrum_po_ICA.powspctrm));
Rel_POW_spektrum_po_ICA_frontal = POW_spektrum_mikrosakada_po_ICA_frontal(zvolena_frekvencni_oblast);%./sum(sum(POW_spektrum_mikrosakada_po_ICA.powspctrm));
Rel_POW_spektrum_pred_ICA_frontal = POW_spektrum_mikrosakada_pred_ICA_frontal(zvolena_frekvencni_oblast);%./sum(sum(POW_spektrum_mikrosakada_pred_ICA.powspctrm));

figure(20)
plot(POW_spektrum_pred_ICA.freq(zvolena_frekvencni_oblast),Rel_POW_spektrum_baseline_pred_ICA_frontal)%-stredni_hodnota_baseline)
title('Absolutni POW - segment neobsahujici mikrosakadicky artefakt pred ICA')
ylabel('Absolutni vykon (\muV^2/Hz)')
xlabel('Frekvence (Hz)')

figure(21)
plot(POW_spektrum_po_ICA.freq(zvolena_frekvencni_oblast),Rel_POW_spektrum_baseline_po_ICA_frontal)%-stredni_hodnota_baseline)
title('Absolutni POW - segment neobsahujici mikrosakadicky artefakt po ICA')
ylabel('Absolutni vykon (\muV^2/Hz)')
xlabel('Frekvence (Hz)')

figure(22)
plot(POW_spektrum_mikrosakada_po_ICA.freq(zvolena_frekvencni_oblast),Rel_POW_spektrum_po_ICA_frontal)
title('Absolutni POW - segment obsahujici mikrosakadicky artefakt po ICA')
ylabel('Absolutni vykon (\muV^2/Hz)')
xlabel('Frekvence (Hz)')

figure(23)
plot(POW_spektrum_mikrosakada_pred_ICA.freq(zvolena_frekvencni_oblast),Rel_POW_spektrum_pred_ICA_frontal)
title('Absolutni POW - segment obsahujici mikrosakadicky artefakt pred ICA')
ylabel('Absolutni vykon (\muV^2/Hz)')
xlabel('Frekvence (Hz)')

figure(24)
subplot(4,1,1)
plot(POW_spektrum_pred_ICA.freq(zvolena_frekvencni_oblast),Rel_POW_spektrum_baseline_pred_ICA_frontal,'r')
title('Relativni POW - baseline pred ICA')
subplot(4,1,2)
plot(POW_spektrum_po_ICA.freq(zvolena_frekvencni_oblast),Rel_POW_spektrum_baseline_po_ICA_frontal,'b')
title('Relativni POW - baseline po ICA')
subplot(4,1,3)
plot(POW_spektrum_mikrosakada_pred_ICA.freq(zvolena_frekvencni_oblast),Rel_POW_spektrum_pred_ICA_frontal,'r')
title('Relativni POW - segment s MS pred ICA')
subplot(4,1,4)
plot(POW_spektrum_mikrosakada_po_ICA.freq(zvolena_frekvencni_oblast),Rel_POW_spektrum_po_ICA_frontal,'b')
title('Relativni POW - segment s MS po ICA')

%% ================= RMSE - baseline, MS po ICA, MS pred ICA ==============
% RMSE = root mean squar error - odmocnina stredn kvadraticke odchylky

% Baseline a segmenty s mikrosakadickm artefaktem po ICA
RMSE_pred_ICA = sqrt(mean(Rel_POW_spektrum_pred_ICA_frontal-Rel_POW_spektrum_baseline_pred_ICA_frontal).^2);

% Baseline a segmenty s mikrosakadickm artefaktem pred ICA
RMSE_po_ICA = sqrt(mean(Rel_POW_spektrum_po_ICA_frontal-Rel_POW_spektrum_baseline_po_ICA_frontal).^2);

disp('Vypocteno RMSE')

%% ====================== Porovnani RMSE v segmentech =====================
% Zobrazeni v hodnot v prikazovem radku

Porovnani_pred_ICA = (sum(POW_spektrum_mikrosakada_pred_ICA_frontal) - sum(Rel_POW_spektrum_baseline_pred_ICA_frontal)) / (sum(POW_spektrum_mikrosakada_pred_ICA_frontal) + sum(Rel_POW_spektrum_baseline_pred_ICA_frontal))*100;
fprintf('Pokles vykonoveho spektra po ICA o %d\n',Porovnani_pred_ICA)

Porovnani_po_ICA = (sum(POW_spektrum_mikrosakada_po_ICA_frontal) - sum(Rel_POW_spektrum_baseline_po_ICA_frontal)) / (sum(POW_spektrum_mikrosakada_po_ICA_frontal) + sum(Rel_POW_spektrum_baseline_po_ICA_frontal))*100;
fprintf('Pokles vykonoveho spektra po ICA o %d\n',Porovnani_po_ICA)


