clc;
clear all;
close all;

%% ========================== Vstupni promenne ============================
% XDF soubor nahrany LSL - LabRecorder
soubor_dat = load_xdf('Mrkani2.xdf');

% Nastaveno, TESTOVANO pro kanaly 18, 25, 32, 37 -> totozne vysledky
volba_kanalu = 18; 

%% ===================== Rozdeleni dat XDF souboru ========================
% EyeTracker
EyeTracker_zaznam = soubor_dat{1,2};
EyeTracker_data = soubor_dat{1,2}.time_series;

% rozdeleni 2_EEG
EEG_zaznam = soubor_dat{1,1};

%% ======================= Predzpracovani dat EEG =========================
% vytvoreni hlavicky EEG souboru pro FieldTrip
data = Uprava_EEG(soubor_dat,1);

% FieldTrip - cfg
cfg = [];
cfg.demean = 'yes';
cfg.detrend = 'yes';
cfg.lpfilttype = 'firws';
cfg.lpfilter = 'yes';
cfg.lpfreq = 10;
cfg.hpfilttype = 'firws';
cfg.hpfilter = 'yes';
cfg.hpfreq = 1.5;
cfg.dftfilter = 'yes';

% Predzpracovani EEG
EEG_data_preprocessing = ft_preprocessing(cfg,data);

disp('Predzpracovani EEG dat HOTOVO...')

%% ====================== Zpracovani dat EyeTracker =======================
% Urceni nahravaneho OKA
if (sum(EyeTracker_data(3,1:20))==0 || sum(EyeTracker_data(4,1:20))==0)
    EyeTracker_X = EyeTracker_data(1,:);
    EyeTracker_Y = EyeTracker_data(2,:);
    disp('Vybrano LEVE oko...')
else
    EyeTracker_X = EyeTracker_data(3,:);
    EyeTracker_Y = EyeTracker_data(4,:);
    disp('Vybrano PRAVE oko...')
end

% Nastaveni prvni hodnoty na 0
EyeTracker_X(1,1) = 0;
EyeTracker_Y(1,1) = 0;

% Nalezeni slepych hodnot v EyeTracker zaznamu
EyeTracker_hodnota_mimo = find(EyeTracker_X <= -32000);
EyeTracker_uprava = 0;
for i=1:length(EyeTracker_hodnota_mimo)
    EyeTracker_uprava(1,EyeTracker_hodnota_mimo(1,i)) = 100;
end

EyeTracker_mimo_uprava = zeros(1,length(EyeTracker_hodnota_mimo)+1);
EyeTracker_mimo_uprava(1,1) = EyeTracker_hodnota_mimo(1,1);
EyeTracker_mimo_uprava(1,2:end) = EyeTracker_hodnota_mimo;

%% =================== Ciselna validace synchronizace =====================
koeficienty = hilbert(EEG_data_preprocessing.trial{1, 1}(18,:));
faze_EEG_signalu = angle(koeficienty);

rozdil = diff(EyeTracker_uprava);
maxim = max(rozdil);
minim = min(rozdil);
start_faze = find(rozdil==minim);
konec_faze = find(rozdil==maxim);

start_konec_faze = [];
start_konec_faze (1,2:length(start_faze)+1) = start_faze;
start_konec_faze (2,1:length(konec_faze)) = konec_faze;
Delsi_useky = [];

j=1;

for i=2:length(start_konec_faze)
    if start_konec_faze(2,i)-start_konec_faze(1,i) > 100
        Delsi_useky(1,j) = start_konec_faze(1,i);
        Delsi_useky(2,j) = start_konec_faze(2,i);
        j = j + 1;
    end
end

Delsi_useky = round(Delsi_useky./soubor_dat{1, 2}.info.effective_srate*1000);

faze_mrkani_start = [];
faze_mrkani_stop = [];
delka_useku = 50;

for i = 1:length(Delsi_useky(1,:))
    faze_mrkani_start(i,1:delka_useku+1) = faze_EEG_signalu(1,Delsi_useky(1,i)-delka_useku/2:Delsi_useky(1,i)+delka_useku/2);
    faze_mrkani_stop(i,1:delka_useku+1) = faze_EEG_signalu(1,Delsi_useky(2,i)-delka_useku/2:Delsi_useky(2,i)+delka_useku/2);
end

% Urceni dostatecne dlouhych useku mrkani k analyze
vektor_analyza = [2:6 8:19];

faze_start_mean = mean(faze_mrkani_start,2)*360/(2*pi);
faze_start_std = std(faze_start_mean(vektor_analyza));

faze_stop_mean = mean(faze_mrkani_stop,2)*360/(2*pi);
faze_stop_std = std(faze_stop_mean(vektor_analyza));

start_mean = mean(faze_start_mean(vektor_analyza));
stop_mean = mean(faze_stop_mean(vektor_analyza));

%% Vypis hodnot
fprintf('Faze EEG signalu v oblasti zacatku mrkani je (%.2f +- %.2f)\n',start_mean,faze_start_std);
fprintf('Faze EEG signalu v oblasti konce mrkani je (%.2f +- %.2f)\n',stop_mean,faze_stop_std);

%% ============ Grafy syn. pulzu - EEG data a EyeTracker dat ==============
figure (1)
plot (soubor_dat{1,1}.time_stamps,EEG_data_preprocessing.trial{1, 1}(volba_kanalu,:),'r')
hold on
plot(soubor_dat{1,2}.time_stamps,EyeTracker_uprava,'b')
hold off
title('Validace synchroniho nahravani EEG zesilovace a EyeTrackeru')
ylabel('Napeti EEG / Souradnice oka v ose X (ms / -)')
xlabel('Casova rada (ms)')
disp('Vyploceno a porovnano')